@@ -4,185 +4,164 @@
     IMPLICIT NONE
     CONTAINS
     SUBROUTINE invoke_0(cu_fld, p_fld, u_fld, cv_fld, v_fld, z_fld, h_fld)
-      USE fortcl, ONLY: get_num_cmd_queues, get_cmd_queues, get_kernel_by_name
-      USE clfortran
-      USE iso_c_binding
-      TYPE(r2d_field), INTENT(INOUT), TARGET :: cu_fld, p_fld, u_fld, cv_fld, v_fld, z_fld, h_fld
-      INTEGER ystop_3
-      INTEGER ystart_3
-      INTEGER xstop_3
-      INTEGER xstart_3
-      INTEGER ystop_2
-      INTEGER ystart_2
-      INTEGER xstop_2
-      INTEGER xstart_2
-      INTEGER ystop_1
-      INTEGER ystart_1
-      INTEGER xstop_1
-      INTEGER xstart_1
-      INTEGER ystop
-      INTEGER ystart
-      INTEGER xstop
-      INTEGER xstart
-      INTEGER i
+      USE fortcl, ONLY: get_cmd_queues, get_kernel_by_name, get_num_cmd_queues
+      USE clfortran, ONLY: clEnqueueNDRangeKernel, clFinish
+      USE iso_c_binding, ONLY: C_LOC, C_NULL_PTR
+      TYPE(r2d_field), intent(inout) :: cu_fld
+      TYPE(r2d_field), intent(inout) :: p_fld
+      TYPE(r2d_field), intent(inout) :: u_fld
+      TYPE(r2d_field), intent(inout) :: cv_fld
+      TYPE(r2d_field), intent(inout) :: v_fld
+      TYPE(r2d_field), intent(inout) :: z_fld
+      TYPE(r2d_field), intent(inout) :: h_fld
       INTEGER j
-      INTEGER(KIND=c_size_t), TARGET :: localsize_3(2)
-      INTEGER(KIND=c_size_t), TARGET :: globalsize_3(2)
-      INTEGER(KIND=c_intptr_t) h_fld_cl_mem
-      INTEGER(KIND=c_size_t), TARGET :: localsize_2(2)
-      INTEGER(KIND=c_size_t), TARGET :: globalsize_2(2)
-      INTEGER(KIND=c_intptr_t) z_fld_cl_mem
-      INTEGER(KIND=c_size_t), TARGET :: localsize_1(2)
-      INTEGER(KIND=c_size_t), TARGET :: globalsize_1(2)
-      INTEGER(KIND=c_intptr_t) v_fld_cl_mem
-      INTEGER(KIND=c_intptr_t) cv_fld_cl_mem
-      INTEGER(KIND=c_size_t), TARGET :: localsize(2)
-      INTEGER(KIND=c_size_t), TARGET :: globalsize(2)
-      INTEGER(KIND=c_intptr_t) u_fld_cl_mem
-      INTEGER(KIND=c_intptr_t) p_fld_cl_mem
-      INTEGER(KIND=c_intptr_t) cu_fld_cl_mem
-      INTEGER(KIND=c_intptr_t), TARGET, SAVE :: kernel_compute_h_code
-      INTEGER(KIND=c_intptr_t), TARGET, SAVE :: kernel_compute_z_code
-      INTEGER(KIND=c_intptr_t), TARGET, SAVE :: kernel_compute_cv_code
-      INTEGER(KIND=c_intptr_t), TARGET, SAVE :: kernel_compute_cu_code
-      LOGICAL, SAVE :: first_time = .TRUE.
+      INTEGER i
+      INTEGER xstart
+      INTEGER xstop
+      INTEGER ystart
+      INTEGER ystop
+      INTEGER xstart_1
+      INTEGER xstop_1
+      INTEGER ystart_1
+      INTEGER ystop_1
+      INTEGER xstart_2
+      INTEGER xstop_2
+      INTEGER ystart_2
+      INTEGER ystop_2
+      INTEGER xstart_3
+      INTEGER xstop_3
+      INTEGER ystart_3
+      INTEGER ystop_3
+      INTEGER(KIND=c_intptr_t), pointer, save :: cmd_queues(:)
+      LOGICAL, save :: first_time = .true.
       INTEGER ierr
-      INTEGER(KIND=c_intptr_t), POINTER, SAVE :: cmd_queues(:)
-      INTEGER, SAVE :: num_cmd_queues
+      INTEGER(KIND=c_size_t), target :: globalsize(2)
+      INTEGER(KIND=c_size_t), target :: localsize(2)
+      INTEGER(KIND=c_intptr_t), TARGET, SAVE :: kernel_compute_cu_code
+      INTEGER(KIND=c_intptr_t), TARGET, SAVE :: kernel_compute_cv_code
+      INTEGER(KIND=c_intptr_t), TARGET, SAVE :: kernel_compute_z_code
+      INTEGER(KIND=c_intptr_t), TARGET, SAVE :: kernel_compute_h_code
+      INTEGER(KIND=c_intptr_t) cu_fld_cl_mem
+      INTEGER(KIND=c_intptr_t) p_fld_cl_mem
+      INTEGER(KIND=c_intptr_t) u_fld_cl_mem
+      INTEGER(KIND=c_intptr_t) cv_fld_cl_mem
+      INTEGER(KIND=c_intptr_t) v_fld_cl_mem
+      INTEGER(KIND=c_intptr_t) z_fld_cl_mem
+      INTEGER(KIND=c_intptr_t) h_fld_cl_mem
+
+      xstart = cu_fld%internal%xstart
+      xstop = cu_fld%internal%xstop
+      ystart = cu_fld%internal%ystart
+      ystop = cu_fld%internal%ystop
+      xstart_1 = cv_fld%internal%xstart
+      xstop_1 = cv_fld%internal%xstop
+      ystart_1 = cv_fld%internal%ystart
+      ystop_1 = cv_fld%internal%ystop
+      xstart_2 = z_fld%internal%xstart
+      xstop_2 = z_fld%internal%xstop
+      ystart_2 = z_fld%internal%ystart
+      ystop_2 = z_fld%internal%ystop
+      xstart_3 = h_fld%internal%xstart
+      xstop_3 = h_fld%internal%xstop
+      ystart_3 = h_fld%internal%ystart
+      ystop_3 = h_fld%internal%ystop
+      ! Initialise OpenCL runtime, kernels and buffers
       IF (first_time) THEN
-        first_time = .FALSE.
         CALL psy_init
-        num_cmd_queues = get_num_cmd_queues()
         cmd_queues => get_cmd_queues()
-        kernel_compute_cu_code = get_kernel_by_name("compute_cu_code")
-        kernel_compute_cv_code = get_kernel_by_name("compute_cv_code")
-        kernel_compute_z_code = get_kernel_by_name("compute_z_code")
-        kernel_compute_h_code = get_kernel_by_name("compute_h_code")
+        kernel_compute_cu_code = get_kernel_by_name('compute_cu_code')
+        kernel_compute_cv_code = get_kernel_by_name('compute_cv_code')
+        kernel_compute_z_code = get_kernel_by_name('compute_z_code')
+        kernel_compute_h_code = get_kernel_by_name('compute_h_code')
         CALL initialise_device_buffer(cu_fld)
         CALL initialise_device_buffer(p_fld)
         CALL initialise_device_buffer(u_fld)
-        xstart = cu_fld % internal % xstart
-        xstop = cu_fld % internal % xstop
-        ystart = cu_fld % internal % ystart
-        ystop = cu_fld % internal % ystop
-        cu_fld_cl_mem = TRANSFER(cu_fld % device_ptr, cu_fld_cl_mem)
-        p_fld_cl_mem = TRANSFER(p_fld % device_ptr, p_fld_cl_mem)
-        u_fld_cl_mem = TRANSFER(u_fld % device_ptr, u_fld_cl_mem)
+        CALL initialise_device_buffer(cv_fld)
+        CALL initialise_device_buffer(v_fld)
+        CALL initialise_device_buffer(z_fld)
+        CALL initialise_device_buffer(h_fld)
+        ! Do a set_args now so subsequent writes place the data appropriately
+        cu_fld_cl_mem = TRANSFER(cu_fld%device_ptr, cu_fld_cl_mem)
+        p_fld_cl_mem = TRANSFER(p_fld%device_ptr, p_fld_cl_mem)
+        u_fld_cl_mem = TRANSFER(u_fld%device_ptr, u_fld_cl_mem)
         CALL compute_cu_code_set_args(kernel_compute_cu_code, cu_fld_cl_mem, p_fld_cl_mem, u_fld_cl_mem, xstart - 1, xstop - 1, &
 &ystart - 1, ystop - 1)
-        CALL cu_fld % write_to_device
-        CALL p_fld % write_to_device
-        CALL u_fld % write_to_device
-        CALL initialise_device_buffer(cv_fld)
-        CALL initialise_device_buffer(p_fld)
-        CALL initialise_device_buffer(v_fld)
-        xstart_1 = cv_fld % internal % xstart
-        xstop_1 = cv_fld % internal % xstop
-        ystart_1 = cv_fld % internal % ystart
-        ystop_1 = cv_fld % internal % ystop
-        cv_fld_cl_mem = TRANSFER(cv_fld % device_ptr, cv_fld_cl_mem)
-        p_fld_cl_mem = TRANSFER(p_fld % device_ptr, p_fld_cl_mem)
-        v_fld_cl_mem = TRANSFER(v_fld % device_ptr, v_fld_cl_mem)
+        cv_fld_cl_mem = TRANSFER(cv_fld%device_ptr, cv_fld_cl_mem)
+        p_fld_cl_mem = TRANSFER(p_fld%device_ptr, p_fld_cl_mem)
+        v_fld_cl_mem = TRANSFER(v_fld%device_ptr, v_fld_cl_mem)
         CALL compute_cv_code_set_args(kernel_compute_cv_code, cv_fld_cl_mem, p_fld_cl_mem, v_fld_cl_mem, xstart_1 - 1, &
 &xstop_1 - 1, ystart_1 - 1, ystop_1 - 1)
-        CALL cv_fld % write_to_device
-        CALL p_fld % write_to_device
-        CALL v_fld % write_to_device
-        CALL initialise_device_buffer(z_fld)
-        CALL initialise_device_buffer(p_fld)
-        CALL initialise_device_buffer(u_fld)
-        CALL initialise_device_buffer(v_fld)
-        xstart_2 = z_fld % internal % xstart
-        xstop_2 = z_fld % internal % xstop
-        ystart_2 = z_fld % internal % ystart
-        ystop_2 = z_fld % internal % ystop
-        z_fld_cl_mem = TRANSFER(z_fld % device_ptr, z_fld_cl_mem)
-        p_fld_cl_mem = TRANSFER(p_fld % device_ptr, p_fld_cl_mem)
-        u_fld_cl_mem = TRANSFER(u_fld % device_ptr, u_fld_cl_mem)
-        v_fld_cl_mem = TRANSFER(v_fld % device_ptr, v_fld_cl_mem)
-        CALL compute_z_code_set_args(kernel_compute_z_code, z_fld_cl_mem, p_fld_cl_mem, u_fld_cl_mem, v_fld_cl_mem, &
-&p_fld % grid % dx, p_fld % grid % dy, xstart_2 - 1, xstop_2 - 1, ystart_2 - 1, ystop_2 - 1)
-        CALL z_fld % write_to_device
-        CALL p_fld % write_to_device
-        CALL u_fld % write_to_device
-        CALL v_fld % write_to_device
-        CALL initialise_device_buffer(h_fld)
-        CALL initialise_device_buffer(p_fld)
-        CALL initialise_device_buffer(u_fld)
-        CALL initialise_device_buffer(v_fld)
-        xstart_3 = h_fld % internal % xstart
-        xstop_3 = h_fld % internal % xstop
-        ystart_3 = h_fld % internal % ystart
-        ystop_3 = h_fld % internal % ystop
-        h_fld_cl_mem = TRANSFER(h_fld % device_ptr, h_fld_cl_mem)
-        p_fld_cl_mem = TRANSFER(p_fld % device_ptr, p_fld_cl_mem)
-        u_fld_cl_mem = TRANSFER(u_fld % device_ptr, u_fld_cl_mem)
-        v_fld_cl_mem = TRANSFER(v_fld % device_ptr, v_fld_cl_mem)
+        z_fld_cl_mem = TRANSFER(z_fld%device_ptr, z_fld_cl_mem)
+        p_fld_cl_mem = TRANSFER(p_fld%device_ptr, p_fld_cl_mem)
+        u_fld_cl_mem = TRANSFER(u_fld%device_ptr, u_fld_cl_mem)
+        v_fld_cl_mem = TRANSFER(v_fld%device_ptr, v_fld_cl_mem)
+        CALL compute_z_code_set_args(kernel_compute_z_code, z_fld_cl_mem, p_fld_cl_mem, u_fld_cl_mem, v_fld_cl_mem, p_fld%grid%dx, &
+&p_fld%grid%dy, xstart_2 - 1, xstop_2 - 1, ystart_2 - 1, ystop_2 - 1)
+        h_fld_cl_mem = TRANSFER(h_fld%device_ptr, h_fld_cl_mem)
+        p_fld_cl_mem = TRANSFER(p_fld%device_ptr, p_fld_cl_mem)
+        u_fld_cl_mem = TRANSFER(u_fld%device_ptr, u_fld_cl_mem)
+        v_fld_cl_mem = TRANSFER(v_fld%device_ptr, v_fld_cl_mem)
         CALL compute_h_code_set_args(kernel_compute_h_code, h_fld_cl_mem, p_fld_cl_mem, u_fld_cl_mem, v_fld_cl_mem, xstart_3 - 1, &
 &xstop_3 - 1, ystart_3 - 1, ystop_3 - 1)
-        CALL h_fld % write_to_device
-        CALL p_fld % write_to_device
-        CALL u_fld % write_to_device
-        CALL v_fld % write_to_device
+        ! Write data to the device
+        CALL cu_fld%write_to_device
+        CALL h_fld%write_to_device
+        CALL v_fld%write_to_device
+        CALL u_fld%write_to_device
+        CALL cv_fld%write_to_device
+        CALL p_fld%write_to_device
+        CALL z_fld%write_to_device
       END IF
-      xstart = cu_fld % internal % xstart
-      xstop = cu_fld % internal % xstop
-      ystart = cu_fld % internal % ystart
-      ystop = cu_fld % internal % ystop
-      cu_fld_cl_mem = TRANSFER(cu_fld % device_ptr, cu_fld_cl_mem)
-      p_fld_cl_mem = TRANSFER(p_fld % device_ptr, p_fld_cl_mem)
-      u_fld_cl_mem = TRANSFER(u_fld % device_ptr, u_fld_cl_mem)
+      globalsize = (/p_fld%grid%nx, p_fld%grid%ny/)
+      localsize = (/4, 1/)
+      cu_fld_cl_mem = TRANSFER(cu_fld%device_ptr, cu_fld_cl_mem)
+      p_fld_cl_mem = TRANSFER(p_fld%device_ptr, p_fld_cl_mem)
+      u_fld_cl_mem = TRANSFER(u_fld%device_ptr, u_fld_cl_mem)
       CALL compute_cu_code_set_args(kernel_compute_cu_code, cu_fld_cl_mem, p_fld_cl_mem, u_fld_cl_mem, xstart - 1, xstop - 1, &
 &ystart - 1, ystop - 1)
-      globalsize = (/p_fld % grid % nx, p_fld % grid % ny/)
-      localsize = (/4, 1/)
+      ! Launch the kernel
       ierr = clEnqueueNDRangeKernel(cmd_queues(1), kernel_compute_cu_code, 2, C_NULL_PTR, C_LOC(globalsize), C_LOC(localsize), 0, &
 &C_NULL_PTR, C_NULL_PTR)
-      xstart_1 = cv_fld % internal % xstart
-      xstop_1 = cv_fld % internal % xstop
-      ystart_1 = cv_fld % internal % ystart
-      ystop_1 = cv_fld % internal % ystop
-      cv_fld_cl_mem = TRANSFER(cv_fld % device_ptr, cv_fld_cl_mem)
-      p_fld_cl_mem = TRANSFER(p_fld % device_ptr, p_fld_cl_mem)
-      v_fld_cl_mem = TRANSFER(v_fld % device_ptr, v_fld_cl_mem)
+      globalsize = (/p_fld%grid%nx, p_fld%grid%ny/)
+      localsize = (/4, 1/)
+      cv_fld_cl_mem = TRANSFER(cv_fld%device_ptr, cv_fld_cl_mem)
+      p_fld_cl_mem = TRANSFER(p_fld%device_ptr, p_fld_cl_mem)
+      v_fld_cl_mem = TRANSFER(v_fld%device_ptr, v_fld_cl_mem)
       CALL compute_cv_code_set_args(kernel_compute_cv_code, cv_fld_cl_mem, p_fld_cl_mem, v_fld_cl_mem, xstart_1 - 1, xstop_1 - 1, &
 &ystart_1 - 1, ystop_1 - 1)
-      globalsize_1 = (/p_fld % grid % nx, p_fld % grid % ny/)
-      localsize_1 = (/4, 1/)
-      ierr = clEnqueueNDRangeKernel(cmd_queues(2), kernel_compute_cv_code, 2, C_NULL_PTR, C_LOC(globalsize_1), C_LOC(localsize_1), &
-&0, C_NULL_PTR, C_NULL_PTR)
-      xstart_2 = z_fld % internal % xstart
-      xstop_2 = z_fld % internal % xstop
-      ystart_2 = z_fld % internal % ystart
-      ystop_2 = z_fld % internal % ystop
-      z_fld_cl_mem = TRANSFER(z_fld % device_ptr, z_fld_cl_mem)
-      p_fld_cl_mem = TRANSFER(p_fld % device_ptr, p_fld_cl_mem)
-      u_fld_cl_mem = TRANSFER(u_fld % device_ptr, u_fld_cl_mem)
-      v_fld_cl_mem = TRANSFER(v_fld % device_ptr, v_fld_cl_mem)
-      CALL compute_z_code_set_args(kernel_compute_z_code, z_fld_cl_mem, p_fld_cl_mem, u_fld_cl_mem, v_fld_cl_mem, &
-&p_fld % grid % dx, p_fld % grid % dy, xstart_2 - 1, xstop_2 - 1, ystart_2 - 1, ystop_2 - 1)
-      globalsize_2 = (/p_fld % grid % nx, p_fld % grid % ny/)
-      localsize_2 = (/4, 1/)
-      ierr = clEnqueueNDRangeKernel(cmd_queues(3), kernel_compute_z_code, 2, C_NULL_PTR, C_LOC(globalsize_2), C_LOC(localsize_2), &
-&0, C_NULL_PTR, C_NULL_PTR)
-      xstart_3 = h_fld % internal % xstart
-      xstop_3 = h_fld % internal % xstop
-      ystart_3 = h_fld % internal % ystart
-      ystop_3 = h_fld % internal % ystop
-      h_fld_cl_mem = TRANSFER(h_fld % device_ptr, h_fld_cl_mem)
-      p_fld_cl_mem = TRANSFER(p_fld % device_ptr, p_fld_cl_mem)
-      u_fld_cl_mem = TRANSFER(u_fld % device_ptr, u_fld_cl_mem)
-      v_fld_cl_mem = TRANSFER(v_fld % device_ptr, v_fld_cl_mem)
+      ! Launch the kernel
+      ierr = clEnqueueNDRangeKernel(cmd_queues(2), kernel_compute_cv_code, 2, C_NULL_PTR, C_LOC(globalsize), C_LOC(localsize), 0, &
+&C_NULL_PTR, C_NULL_PTR)
+      globalsize = (/p_fld%grid%nx, p_fld%grid%ny/)
+      localsize = (/4, 1/)
+      z_fld_cl_mem = TRANSFER(z_fld%device_ptr, z_fld_cl_mem)
+      p_fld_cl_mem = TRANSFER(p_fld%device_ptr, p_fld_cl_mem)
+      u_fld_cl_mem = TRANSFER(u_fld%device_ptr, u_fld_cl_mem)
+      v_fld_cl_mem = TRANSFER(v_fld%device_ptr, v_fld_cl_mem)
+      CALL compute_z_code_set_args(kernel_compute_z_code, z_fld_cl_mem, p_fld_cl_mem, u_fld_cl_mem, v_fld_cl_mem, p_fld%grid%dx, &
+&p_fld%grid%dy, xstart_2 - 1, xstop_2 - 1, ystart_2 - 1, ystop_2 - 1)
+      ! Launch the kernel
+      ierr = clEnqueueNDRangeKernel(cmd_queues(3), kernel_compute_z_code, 2, C_NULL_PTR, C_LOC(globalsize), C_LOC(localsize), 0, &
+&C_NULL_PTR, C_NULL_PTR)
+      globalsize = (/p_fld%grid%nx, p_fld%grid%ny/)
+      localsize = (/4, 1/)
+      h_fld_cl_mem = TRANSFER(h_fld%device_ptr, h_fld_cl_mem)
+      p_fld_cl_mem = TRANSFER(p_fld%device_ptr, p_fld_cl_mem)
+      u_fld_cl_mem = TRANSFER(u_fld%device_ptr, u_fld_cl_mem)
+      v_fld_cl_mem = TRANSFER(v_fld%device_ptr, v_fld_cl_mem)
       CALL compute_h_code_set_args(kernel_compute_h_code, h_fld_cl_mem, p_fld_cl_mem, u_fld_cl_mem, v_fld_cl_mem, xstart_3 - 1, &
 &xstop_3 - 1, ystart_3 - 1, ystop_3 - 1)
-      globalsize_3 = (/p_fld % grid % nx, p_fld % grid % ny/)
-      localsize_3 = (/4, 1/)
-      ierr = clEnqueueNDRangeKernel(cmd_queues(4), kernel_compute_h_code, 2, C_NULL_PTR, C_LOC(globalsize_3), C_LOC(localsize_3), &
-&0, C_NULL_PTR, C_NULL_PTR)
+      ! Launch the kernel
+      ierr = clEnqueueNDRangeKernel(cmd_queues(4), kernel_compute_h_code, 2, C_NULL_PTR, C_LOC(globalsize), C_LOC(localsize), 0, &
+&C_NULL_PTR, C_NULL_PTR)
+      ! Wait until all kernels have finished
       ierr = clFinish(cmd_queues(1))
       ierr = clFinish(cmd_queues(2))
       ierr = clFinish(cmd_queues(3))
       ierr = clFinish(cmd_queues(4))
+      ! Unset the first time flag
+      first_time = .false.
+
     END SUBROUTINE invoke_0
     SUBROUTINE psy_init()
       USE fortcl, ONLY: add_kernels, ocl_env_init
@@ -194,9 +173,9 @@
         initialised = .true.
         CALL ocl_env_init(4, ocl_device_num, .false., .false.)
         kernel_names(1) = 'compute_cv_code'
-        kernel_names(2) = 'compute_cu_code'
+        kernel_names(2) = 'compute_z_code'
         kernel_names(3) = 'compute_h_code'
-        kernel_names(4) = 'compute_z_code'
+        kernel_names(4) = 'compute_cu_code'
         CALL add_kernels(4, kernel_names)
       END IF
 
