@@ -49,10 +49,10 @@ C representation of the PSyIR.
 from __future__ import print_function
 from psyclone.psyir.nodes import Reference, Literal, UnaryOperation, \
     BinaryOperation, NaryOperation, Assignment, IfBlock, Loop, \
-    Container, Range, ArrayReference, Call, KernelSchedule
+    Container, Range, ArrayReference, Call, Routine, FileContainer
 from psyclone.psyir.symbols import DataSymbol, RoutineSymbol, SymbolTable, \
     ContainerSymbol, ArgumentInterface, ScalarType, ArrayType, \
-    GlobalInterface, REAL_TYPE, REAL4_TYPE, REAL_DOUBLE_TYPE, INTEGER_TYPE, \
+    ImportInterface, REAL_TYPE, REAL4_TYPE, REAL_DOUBLE_TYPE, INTEGER_TYPE, \
     INTEGER_SINGLE_TYPE, INTEGER4_TYPE, INTEGER8_TYPE
 from psyclone.psyir.backend.fortran import FortranWriter
 from psyclone.psyir.backend.c import CWriter
@@ -151,24 +151,49 @@ def create_psyir_tree():
     loop = Loop.create(index_symbol, int_zero(), int_one(), int_one(),
                        [ifblock])
 
-    # KernelSchedule
-    kernel_schedule = KernelSchedule.create(
+    # Routine
+    routine = Routine.create(
         "work", symbol_table, [assign1, call, assign2, loop, assign5, assign6])
 
     # Container
     container_symbol_table = SymbolTable()
     container = Container.create("CONTAINER", container_symbol_table,
-                                 [kernel_schedule])
+                                 [routine])
+
+    # Container, Routines and any statement can have comments
+    container.preceding_comment = "PSyIR Node creation example"
+    routine.preceding_comment = "Example work routine"
+    call.preceding_comment = "Any statement can have preceding ..."
+    call.inline_comment = " ... and inline comments."
 
     # Import data from another container
     external_container = ContainerSymbol("some_mod")
     container_symbol_table.add(external_container)
     external_var = DataSymbol("some_var", INTEGER_TYPE,
-                              interface=GlobalInterface(external_container))
+                              interface=ImportInterface(external_container))
     container_symbol_table.add(external_var)
-    routine_symbol.interface = GlobalInterface(external_container)
+    routine_symbol.interface = ImportInterface(external_container)
     container_symbol_table.add(routine_symbol)
-    return container
+
+    # Routine (specified as being a program)
+    program_symbol_table = SymbolTable()
+    work_symbol = RoutineSymbol("work")
+    container_symbol = ContainerSymbol("CONTAINER")
+    work_symbol.interface = ImportInterface(container_symbol)
+    arg_symbol = program_symbol_table.new_symbol(root_name="arg",
+                                                 symbol_type=DataSymbol,
+                                                 datatype=REAL_TYPE)
+    program_symbol_table.add(container_symbol)
+    program_symbol_table.add(work_symbol)
+    call = Call.create(work_symbol, [Reference(arg_symbol)])
+    program = Routine.create(
+        "some_program", program_symbol_table, [call], is_program=True)
+
+    # File container
+    file_container = FileContainer.create(
+        "dummy", SymbolTable(), [container, program])
+
+    return file_container
 
 
 if __name__ == "__main__":
@@ -179,9 +204,9 @@ if __name__ == "__main__":
     result = writer(psyir_tree)
     print(result)
 
-    # Write out the code as C. At the moment NaryOperator, KernelSchedule
+    # Write out the code as C. At the moment NaryOperator, Routine
     # and Container are not supported in the C backend so the full example
     # can't be output.
     writer = CWriter()
-    result = writer(psyir_tree.children[0].children[3])
+    result = writer(psyir_tree.children[0].children[0].children[3])
     print(result)
