@@ -1,7 +1,7 @@
 # -----------------------------------------------------------------------------
 # BSD 3-Clause License
 #
-# Copyright (c) 2018-2021, Science and Technology Facilities Council.
+# Copyright (c) 2018-2022, Science and Technology Facilities Council.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -97,15 +97,14 @@ class Config(object):
     _instance = None
 
     # List of supported API by PSyclone
-    _supported_api_list = ["dynamo0.1", "dynamo0.3",
-                           "gocean0.1", "gocean1.0", "nemo"]
+    _supported_api_list = ["dynamo0.3", "gocean1.0", "nemo"]
 
     # List of supported stub API by PSyclone
     _supported_stub_api_list = ["dynamo0.3"]
 
     # The default API, i.e. the one to be used if neither a command line
     # option is specified nor is the API in the config file used.
-    _default_api = u"dynamo0.3"
+    _default_api = "dynamo0.3"
 
     # The default scheme to use when (re)naming transformed kernels.
     # By default we support multiple, different versions of any given
@@ -241,10 +240,10 @@ class Config(object):
         # Check for missing section headers and general parsing errors
         # (e.g. incomplete or incorrect key-value mapping)
         except (MissingSectionHeaderError, ParsingError) as err:
-            raise ConfigurationError(
+            six.raise_from(ConfigurationError(
                 "ConfigParser failed to read the configuration file. Is it "
                 "formatted correctly? (Error was: {0})".format(str(err)),
-                config=self)
+                config=self), err)
 
         # Check that the configuration file has a [DEFAULT] section. Even
         # if there isn't one in the file, ConfigParser creates an (empty)
@@ -264,13 +263,13 @@ class Config(object):
             self._distributed_mem = self._config['DEFAULT'].getboolean(
                 'DISTRIBUTED_MEMORY')
         except ValueError as err:
-            raise ConfigurationError(
+            six.raise_from(ConfigurationError(
                 "Error while parsing DISTRIBUTED_MEMORY: {0}".
-                format(str(err)), config=self)
+                format(str(err)), config=self), err)
 
-        # API for psyclone
-        if "API" in self._config["DEFAULT"]:
-            self._api = self._config['DEFAULT']['API']
+        # API for PSyclone
+        if "DEFAULTAPI" in self._config["DEFAULT"]:
+            self._api = self._config['DEFAULT']['DEFAULTAPI']
         else:
             self._api = Config._default_api
             # Test if we have exactly one section (besides DEFAULT).
@@ -310,17 +309,17 @@ class Config(object):
             self._reproducible_reductions = self._config['DEFAULT'].getboolean(
                 'REPRODUCIBLE_REDUCTIONS')
         except ValueError as err:
-            raise ConfigurationError(
+            six.raise_from(ConfigurationError(
                 "Error while parsing REPRODUCIBLE_REDUCTIONS: {0}".
-                format(str(err)), config=self)
+                format(str(err)), config=self), err)
 
         try:
             self._reprod_pad_size = self._config['DEFAULT'].getint(
                 'REPROD_PAD_SIZE')
         except ValueError as err:
-            raise ConfigurationError(
+            six.raise_from(ConfigurationError(
                 "error while parsing REPROD_PAD_SIZE: {0}".format(str(err)),
-                config=self)
+                config=self), err)
 
         if 'PSYIR_ROOT_NAME' not in self._config['DEFAULT']:
             # Use the default name if no default is specified for the
@@ -360,18 +359,7 @@ class Config(object):
         for api in Config._supported_api_list:
             if api in self._config:
                 if api == "dynamo0.3":
-                    self._api_conf[api] = DynConfig(self, self._config[api])
-                elif api == "dynamo0.1":
-                    # For now we use the same class as dynamo0.3.
-                    # However, we use it to read a different section of the
-                    # config file, so the dynamo0.1 mapping will be correctly
-                    # used.
-                    self._api_conf[api] = DynConfig(self, self._config[api])
-                elif api == "gocean0.1":
-                    # For now we use the same class as gocean1.0.
-                    # However, we use it to read a different section of the
-                    # config file
-                    self._api_conf[api] = GOceanConfig(self, self._config[api])
+                    self._api_conf[api] = LFRicConfig(self, self._config[api])
                 elif api == "gocean1.0":
                     self._api_conf[api] = GOceanConfig(self, self._config[api])
                 elif api == "nemo":
@@ -395,7 +383,7 @@ class Config(object):
                 required. If none is specified, returns the config for the
                 default API.
         :returns: object containing API-specific configuration
-        :rtype: One of :py:class:`psyclone.configuration.DynConfig`,
+        :rtype: One of :py:class:`psyclone.configuration.LFRicConfig`,
                 :py:class:`psyclone.configuration.GOceanConfig` or None.
 
         :raises ConfigurationError: if api is not in the list of supported \
@@ -670,9 +658,10 @@ class Config(object):
                     raise ConfigurationError(
                         "Include path '{0}' does not exist".format(path))
                 self._include_paths.append(path)
-        except (TypeError, ValueError):
-            raise ValueError("include_paths must be a list but got: {0}".
-                             format(type(path_list)))
+        except (TypeError, ValueError) as err:
+            six.raise_from(ValueError("include_paths must be a list but got: "
+                                      "{0}". format(type(path_list))),
+                           err)
 
     @property
     def valid_psy_data_prefixes(self):
@@ -692,9 +681,16 @@ class Config(object):
         '''
         return self._config.defaults()
 
+    def get_constants(self):
+        ''':returns: the constants instance of the current API.
+        :rtype: either :py:class:`psyclone.domain.lfric.LFRicConstants`, \
+            :py:class:`psyclone.domain.gocean.GOceanConstants`, or \
+            :py:class:`psyclone.domain.nemo.NemoConstants`
+        '''
+        return self.api_conf().get_constants()
+
 
 # =============================================================================
-# TODO (issue #1163): Add support for lists in the configuration file
 class APISpecificConfig(object):
     '''A base class for functions that each API-specific class must provide.
     At the moment this is just the function 'access_mapping' that maps between
@@ -730,12 +726,13 @@ class APISpecificConfig(object):
             try:
                 self._access_mapping[api_access_name] = \
                     AccessType.from_string(access_type)
-            except ValueError:
+            except ValueError as err:
                 # Raised by from_string()
-                raise ConfigurationError("Unknown access type '{0}' found "
-                                         "for key '{1}'"
-                                         .format(access_type,
-                                                 api_access_name))
+                six.raise_from(ConfigurationError("Unknown access type '{0}' "
+                                                  "found for key '{1}'"
+                                                  .format(access_type,
+                                                          api_access_name)),
+                               err)
 
         # Now create the reverse lookup (for better error messages):
         self._reverse_access_mapping = {v: k for k, v in
@@ -762,10 +759,12 @@ class APISpecificConfig(object):
         for entry in input_list:
             try:
                 key, value = entry.split(":", 1)
-            except ValueError:
+            except ValueError as err:
                 # Raised when split does not return two elements:
-                raise ConfigurationError("Invalid format for mapping: {0}".
-                                         format(entry.strip()))
+                six.raise_from(ConfigurationError("Invalid format for "
+                                                  "mapping: {0}".
+                                                  format(entry.strip())),
+                               err)
             # Remove spaces and convert unicode to normal strings in Python2
             return_dict[str(key.strip())] = str(value.strip())
         return return_dict
@@ -807,10 +806,10 @@ class APISpecificConfig(object):
 
 
 # =============================================================================
-class DynConfig(APISpecificConfig):
+class LFRicConfig(APISpecificConfig):
     '''
     LFRic-specific (Dynamo 0.3) Config sub-class. Holds configuration options
-    specific to the LFRic (Dynamo 0.3) API and Dynamo 0.1 API.
+    specific to the LFRic (Dynamo 0.3) API.
 
     :param config: the 'parent' Config object.
     :type config: :py:class:`psyclone.configuration.Config`
@@ -837,7 +836,7 @@ class DynConfig(APISpecificConfig):
     '''
     # pylint: disable=too-few-public-methods, too-many-instance-attributes
     def __init__(self, config, section):
-        super(DynConfig, self).__init__(section)
+        super().__init__(section)
         # Ref. to parent Config object
         self._config = config
         # Initialise redundant computation setting
@@ -850,123 +849,111 @@ class DynConfig(APISpecificConfig):
         # Number of ANY_SPACE and ANY_DISCONTINUOUS_SPACE function spaces
         self._num_any_space = None
         self._num_any_discontinuous_space = None
-        # Set mandatory keys
-        # TODO: to be fully populated here for LFRic (Dynamo0.3) API in #282
-        # when Dynamo0.1 API is removed.
-        self._mandatory_keys = []
 
-        # TODO: This "if" clause will become redundant in #282 as there will be
-        # just LFRic (Dynamo0.3) API configuration
-        if section.name == "dynamo0.3":
-            # Define and check mandatory keys
-            self._mandatory_keys = ["access_mapping",
-                                    "compute_annexed_dofs",
-                                    "supported_fortran_datatypes",
-                                    "default_kind",
-                                    "run_time_checks",
-                                    "num_any_space",
-                                    "num_any_discontinuous_space"]
-            mdkeys = set(self._mandatory_keys)
-            if not mdkeys.issubset(set(section.keys())):
-                raise ConfigurationError(
-                    "Missing mandatory configuration option in the '[{0}]' "
-                    "section of the configuration file '{1}'. "
-                    "Valid options are: {2}."
-                    .format(section.name, config.filename,
-                            self._mandatory_keys))
+        # Define and check mandatory keys
+        self._mandatory_keys = ["access_mapping",
+                                "compute_annexed_dofs",
+                                "supported_fortran_datatypes",
+                                "default_kind",
+                                "run_time_checks",
+                                "num_any_space",
+                                "num_any_discontinuous_space"]
+        mdkeys = set(self._mandatory_keys)
+        if not mdkeys.issubset(set(section.keys())):
+            raise ConfigurationError(
+                f"Missing mandatory configuration option in the "
+                f"'[{section.name}]' section of the configuration file "
+                f"'{config.filename}'. Valid options are: "
+                f"{self._mandatory_keys}.")
 
-            # Parse setting for redundant computation over annexed dofs
-            try:
-                self._compute_annexed_dofs = section.getboolean(
-                    "compute_annexed_dofs")
-            except ValueError as err:
-                six.raise_from(ConfigurationError(
-                    "Error while parsing COMPUTE_ANNEXED_DOFS in the "
-                    "'[{0}]' section of the configuration file '{1}': {2}."
-                    .format(section.name, config.filename, str(err)),
-                    config=self._config), err)
+        # Parse setting for redundant computation over annexed dofs
+        try:
+            self._compute_annexed_dofs = section.getboolean(
+                "compute_annexed_dofs")
+        except ValueError as err:
+            raise ConfigurationError(
+                f"Error while parsing COMPUTE_ANNEXED_DOFS in the "
+                f"'[{section.name}]' section of the configuration file "
+                f"'{config.filename}': {str(err)}.",
+                config=self._config) from err
 
-            # Parse setting for run_time_checks flag
-            try:
-                self._run_time_checks = section.getboolean(
-                    "run_time_checks")
-            except ValueError as err:
-                six.raise_from(ConfigurationError(
-                    "Error while parsing RUN_TIME_CHECKS in the '[{0}]' "
-                    "section of the configuration file '{1}': {2}."
-                    .format(section.name, config.filename, str(err)),
-                    config=self._config), err)
+        # Parse setting for run_time_checks flag
+        try:
+            self._run_time_checks = section.getboolean(
+                "run_time_checks")
+        except ValueError as err:
+            raise ConfigurationError(
+                f"Error while parsing RUN_TIME_CHECKS in the "
+                f"'[{section.name}]' section of the configuration file "
+                f"'{config.filename}': {str(err)}.",
+                config=self._config) from err
 
-            # Parse setting for the supported Fortran datatypes. No
-            # need to check whether the keyword is found as it is
-            # mandatory (and therefore already checked).
-            self._supported_fortran_datatypes = section.getlist(
-                "supported_fortran_datatypes")
+        # Parse setting for the supported Fortran datatypes. No
+        # need to check whether the keyword is found as it is
+        # mandatory (and therefore already checked).
+        self._supported_fortran_datatypes = section.getlist(
+            "supported_fortran_datatypes")
 
-            # Parse setting for default kinds (precisions). No need to
-            # check whether the keyword is found as it is mandatory
-            # (and therefore already checked).
-            kind_list = section.getlist("default_kind")
-            all_kinds = self.create_dict_from_list(kind_list)
-            # Set default kinds (precisions) from config file
-            # Check for valid datatypes (filter to remove empty values)
-            datatypes = set(filter(None, all_kinds.keys()))
-            if datatypes != set(self._supported_fortran_datatypes):
-                raise ConfigurationError(
-                    "Fortran datatypes in the 'default_kind' mapping in the "
-                    "'[{0}]' section of the configuration file '{1}' do "
-                    "not match the supported Fortran datatypes {2}."
-                    .format(section.name, config.filename,
-                            self._supported_fortran_datatypes))
-            # Check for valid kinds (filter to remove any empty values)
-            datakinds = set(filter(None, all_kinds.values()))
-            if len(datakinds) != len(set(self._supported_fortran_datatypes)):
-                raise ConfigurationError(
-                    "Supplied kind parameters {0} in the '[{1}]' section "
-                    "of the configuration file '{2}' do not define the "
-                    "default kind for one or more supported datatypes {3}."
-                    .format(sorted(datakinds), section.name, config.filename,
-                            self._supported_fortran_datatypes))
-            self._default_kind = all_kinds
+        # Parse setting for default kinds (precisions). No need to
+        # check whether the keyword is found as it is mandatory
+        # (and therefore already checked).
+        kind_list = section.getlist("default_kind")
+        all_kinds = self.create_dict_from_list(kind_list)
+        # Set default kinds (precisions) from config file
+        # Check for valid datatypes (filter to remove empty values)
+        datatypes = set(filter(None, all_kinds.keys()))
+        if datatypes != set(self._supported_fortran_datatypes):
+            raise ConfigurationError(
+                f"Fortran datatypes in the 'default_kind' mapping in the "
+                f"'[{section.name}]' section of the configuration file "
+                f"'{config.filename}' do not match the supported Fortran "
+                f"datatypes {self._supported_fortran_datatypes}.")
+        # Check for valid kinds (filter to remove any empty values)
+        datakinds = set(filter(None, all_kinds.values()))
+        if len(datakinds) != len(set(self._supported_fortran_datatypes)):
+            raise ConfigurationError(
+                f"Supplied kind parameters {sorted(datakinds)} in the "
+                f"'[{section.name}]' section of the configuration file "
+                f"'{config.filename}' do not define the default kind for "
+                f"one or more supported datatypes "
+                f"{self._supported_fortran_datatypes}.")
+        self._default_kind = all_kinds
 
-            # Parse setting for the number of ANY_SPACE function spaces
-            # (check for an invalid value and numbers <= 0)
-            try:
-                self._num_any_space = section.getint("NUM_ANY_SPACE")
-            except ValueError as err:
-                six.raise_from(ConfigurationError(
-                    "Error while parsing NUM_ANY_SPACE in the '[{0}]' "
-                    "section of the configuration file '{1}': {2}."
-                    .format(section.name, config.filename, str(err)),
-                    config=self._config), err)
+        # Parse setting for the number of ANY_SPACE function spaces
+        # (check for an invalid value and numbers <= 0)
+        try:
+            self._num_any_space = section.getint("NUM_ANY_SPACE")
+        except ValueError as err:
+            raise ConfigurationError(
+                f"Error while parsing NUM_ANY_SPACE in the '[{section.name}]' "
+                f"section of the configuration file '{config.filename}': "
+                f"{str(err)}.", config=self._config) from err
 
-            if self._num_any_space <= 0:
-                raise ConfigurationError(
-                    "The supplied number of ANY_SPACE function spaces "
-                    "in the '[{0}]' section of the configuration "
-                    "file '{1}' must be greater than 0 but found {2}."
-                    .format(section.name, config.filename,
-                            self._num_any_space))
+        if self._num_any_space <= 0:
+            raise ConfigurationError(
+                f"The supplied number of ANY_SPACE function spaces "
+                f"in the '[{section.name}]' section of the configuration "
+                f"file '{config.filename}' must be greater than 0 but found "
+                f"{self._num_any_space}.")
 
-            # Parse setting for the number of ANY_DISCONTINUOUS_SPACE
-            # function spaces (checks for an invalid value and numbers <= 0)
-            try:
-                self._num_any_discontinuous_space = section.getint(
-                    "NUM_ANY_DISCONTINUOUS_SPACE")
-            except ValueError as err:
-                six.raise_from(ConfigurationError(
-                    "Error while parsing NUM_ANY_DISCONTINUOUS_SPACE in the "
-                    "'[{0}]' section of the configuration file '{1}': {2}."
-                    .format(section.name, config.filename, str(err)),
-                    config=self._config), err)
+        # Parse setting for the number of ANY_DISCONTINUOUS_SPACE
+        # function spaces (checks for an invalid value and numbers <= 0)
+        try:
+            self._num_any_discontinuous_space = section.getint(
+                "NUM_ANY_DISCONTINUOUS_SPACE")
+        except ValueError as err:
+            raise ConfigurationError(
+                f"Error while parsing NUM_ANY_DISCONTINUOUS_SPACE in the "
+                f"'[{section.name}]' section of the configuration file "
+                f"'{config.filename}': {str(err)}.",
+                config=self._config) from err
 
-            if self._num_any_discontinuous_space <= 0:
-                raise ConfigurationError(
-                    "The supplied number of ANY_DISCONTINUOUS_SPACE function "
-                    "spaces in the '[{0}]' section of the configuration "
-                    "file '{1}' must be greater than 0 but found {2}."
-                    .format(section.name, config.filename,
-                            self._num_any_discontinuous_space))
+        if self._num_any_discontinuous_space <= 0:
+            raise ConfigurationError(
+                f"The supplied number of ANY_DISCONTINUOUS_SPACE function "
+                f"spaces in the '[{section.name}]' section of the "
+                f"configuration file '{config.filename}' must be greater than "
+                f"0 but found {self._num_any_discontinuous_space}.")
 
     @property
     def compute_annexed_dofs(self):
@@ -1109,7 +1096,7 @@ class GOceanConfig(APISpecificConfig):
                     try:
                         fortran, variable_type, intrinsic_type = \
                             all_props[grid_property].split(":")
-                    except ValueError:
+                    except ValueError as err:
                         # Raised when the string does not contain exactly
                         # three values separated by ":"
                         error = "Invalid property \"{0}\" found with value " \
@@ -1121,7 +1108,7 @@ class GOceanConfig(APISpecificConfig):
                                 .format(grid_property,
                                         all_props[grid_property],
                                         config.filename)
-                        raise ConfigurationError(error)
+                        six.raise_from(ConfigurationError(error), err)
                     # Make sure to remove the spaces which the config
                     # file might contain
                     self._grid_properties[grid_property] = \
@@ -1315,10 +1302,14 @@ class NemoConfig(APISpecificConfig):
 
     def get_valid_loop_types(self):
         '''
-        :returns: a list of valid loop types.
+        The list is sorted to have reproducible results for testing.
+        :returns: a sorted list of valid loop types.
         :rtype: list of str.
+
         '''
-        return list(self._loop_type_data)
+        valid_types_list = list(self._loop_type_data)
+        valid_types_list.sort()
+        return valid_types_list
 
     def get_index_order(self):
         '''
@@ -1345,6 +1336,6 @@ class NemoConfig(APISpecificConfig):
 __all__ = ["APISpecificConfig",
            "Config",
            "ConfigurationError",
-           "DynConfig",
+           "LFRicConfig",
            "GOceanConfig",
            "NemoConfig"]
