@@ -56,9 +56,10 @@ class CreateNemoKernelTrans(Transformation):
     >>> from psyclone.domain.nemo.transformations import CreateNemoKernelTrans
     >>> code = '''
     ... subroutine sub()
-    ...   integer :: ji, tmp(10)
+    ...   integer :: ji
+    ...   real :: tmp(10)
     ...   do ji=1, 10
-    ...     tmp(ji) = 2*ji
+    ...     tmp(ji) = 2.0*ji
     ...   end do
     ... end subroutine sub'''
     >>> psyir = FortranReader().psyir_from_source(code)
@@ -66,20 +67,21 @@ class CreateNemoKernelTrans(Transformation):
     >>> trans = CreateNemoKernelTrans()
     >>> trans.apply(loop.loop_body)
     >>> psyir.view()
-    Routine[name:'sub']
-        0: Loop[type='None', field_space='None', it_space='None']
-            Literal[value:'1', Scalar<INTEGER, UNDEFINED>]
-            Literal[value:'10', Scalar<INTEGER, UNDEFINED>]
-            Literal[value:'1', Scalar<INTEGER, UNDEFINED>]
-            Schedule[]
-                0: InlinedKern[]
-                    Schedule[]
-                        0: Assignment[]
-                            ArrayReference[name:'tmp']
-                                Reference[name:'ji']
-                            BinaryOperation[operator:'MUL']
-                                Literal[value:'2.0', Scalar<REAL, UNDEFINED>]
-                                Reference[name:'ji']
+    FileContainer[None]
+        Routine[name:'sub']
+            0: Loop[type='None', field_space='None', it_space='None']
+                Literal[value:'1', Scalar<INTEGER, UNDEFINED>]
+                Literal[value:'10', Scalar<INTEGER, UNDEFINED>]
+                Literal[value:'1', Scalar<INTEGER, UNDEFINED>]
+                Schedule[]
+                    0: InlinedKern[]
+                        Schedule[]
+                            0: Assignment[]
+                                ArrayReference[name:'tmp']
+                                    Reference[name:'ji']
+                                BinaryOperation[operator:'MUL']
+                                    Literal[value:'2.0', Scalar<REAL, UNDEFINED>]
+                                    Reference[name:'ji']
 
     The resulting Schedule contains a NemoKern (displayed as an
     'InlinedKern' by the view() method).
