@@ -1,7 +1,7 @@
 # -----------------------------------------------------------------------------
 # BSD 3-Clause License
 #
-# Copyright (c) 2020-2021, Science and Technology Facilities Council.
+# Copyright (c) 2020-2022, Science and Technology Facilities Council.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -32,6 +32,7 @@
 # POSSIBILITY OF SUCH DAMAGE.
 # -----------------------------------------------------------------------------
 # Author R. W. Ford, STFC Daresbury Lab
+# Modified S. Siso, STFC Daresbury Lab
 
 '''Module providing a transformation from an Assignment node
 containing an Array Reference node in its left-hand-side which in turn
@@ -44,19 +45,21 @@ array index should be transformed.
 '''
 
 from __future__ import absolute_import
-
-from psyclone.psyGen import Transformation, InvokeSchedule
-from psyclone.psyir.transformations.transformation_error \
-    import TransformationError
-from psyclone.errors import InternalError
-from psyclone.psyir.symbols import DataSymbol, INTEGER_TYPE, DeferredType
-from psyclone.psyir.symbols.datatypes import ScalarType
-from psyclone.psyir.nodes import Range, Reference, ArrayReference, \
-    Assignment, Literal, Operation, BinaryOperation
-from psyclone.nemo import NemoLoop
 from psyclone.configuration import Config
-from psyclone.domain.nemo.transformations.create_nemo_kernel_trans \
-    import CreateNemoKernelTrans
+from psyclone.domain.nemo.transformations.create_nemo_kernel_trans import \
+    CreateNemoKernelTrans
+from psyclone.errors import LazyString, InternalError
+from psyclone.nemo import NemoLoop
+from psyclone.psyGen import Transformation
+from psyclone.psyir.backend.fortran import FortranWriter
+from psyclone.psyir.nodes import Range, Reference, ArrayReference, Call, \
+    Assignment, Literal, Operation, CodeBlock, ArrayMember, Loop, Routine, \
+    BinaryOperation, StructureReference, StructureMember, Node
+from psyclone.psyir.nodes.array_mixin import ArrayMixin
+from psyclone.psyir.symbols import DataSymbol, INTEGER_TYPE, DeferredType, \
+    ScalarType
+from psyclone.psyir.transformations.transformation_error import \
+    TransformationError
 
 
 class NemoArrayRange2LoopTrans(Transformation):
@@ -104,7 +107,7 @@ class NemoArrayRange2LoopTrans(Transformation):
         (with the same loop index) is also placed around the modified
         assignment statement. If the array reference on the
         left-hand-side of the assignment only had one range node as an
-        index (so now has none) then the assigment is also placed
+        index (so now has none) then the assignment is also placed
         within a NemoKern.
 
         The name of the loop index is taken from the PSyclone
@@ -130,11 +133,11 @@ class NemoArrayRange2LoopTrans(Transformation):
         self.validate(node)
 
         array_reference = node.parent
-        array_index = node.position
-        assignment = array_reference.parent
+        array_index = node.parent.indices.index(node)
+        assignment = node.ancestor(Assignment)
         parent = assignment.parent
         # Ensure we always use the routine-level symbol table
-        symbol_table = node.ancestor(InvokeSchedule).symbol_table
+        symbol_table = node.ancestor(Routine).symbol_table
 
         # See if there is any configuration information for this array index
         loop_type_order = Config.get().api_conf("nemo").get_index_order()
@@ -154,6 +157,7 @@ class NemoArrayRange2LoopTrans(Transformation):
             loop_variable_name = symbol_table.next_available_name("idx")
 
         # Lower bound
+        lower_bound = None
         if not array_reference.is_lower_bound(array_index):
             # The range specifies a lower bound so use it
             lower_bound = node.start.copy()
@@ -163,13 +167,19 @@ class NemoArrayRange2LoopTrans(Transformation):
                 _ = int(lower_bound_info)
                 lower_bound = Literal(lower_bound_info, INTEGER_TYPE)
             except ValueError:
-                lower_bound = Reference(symbol_table.lookup(lower_bound_info))
-        else:
-            # The lower bound is not set or specified so use the
-            # LBOUND() intrinsic
+                # It's not a Literal, but is it an existing symbol?
+                try:
+                    lower_bound = Reference(
+                        symbol_table.lookup(lower_bound_info))
+                except KeyError:
+                    # The config lower bound symbol name does not exist
+                    pass
+        # The lower bound is still not set so use the LBOUND() intrinsic
+        if not lower_bound:
             lower_bound = node.start.copy()
 
         # Upper bound
+        upper_bound = None
         if not array_reference.is_upper_bound(array_index):
             # The range specifies an upper bound so use it
             upper_bound = node.stop.copy()
@@ -179,10 +189,15 @@ class NemoArrayRange2LoopTrans(Transformation):
                 _ = int(upper_bound_info)
                 upper_bound = Literal(upper_bound_info, INTEGER_TYPE)
             except ValueError:
-                upper_bound = Reference(symbol_table.lookup(upper_bound_info))
-        else:
-            # The upper bound is not set or specified so use the
-            # UBOUND() intrinsic
+                # It's not a Literal, but is it an existing symbol?
+                try:
+                    upper_bound = Reference(
+                        symbol_table.lookup(upper_bound_info))
+                except KeyError:
+                    # The config upper bound symbol name does not exist
+                    pass
+        # The upper_bound bound is still not set so use the UBOUND() intrinsic
+        if not upper_bound:
             upper_bound = node.stop.copy()
 
         # Just use the specified step value
@@ -190,57 +205,41 @@ class NemoArrayRange2LoopTrans(Transformation):
 
         # Look up the loop variable in the symbol table. If it does
         # not exist then create it.
-        try:
-            loop_variable_symbol = symbol_table.lookup(loop_variable_name)
-        except KeyError:
-            # Add loop variable as it does not already exist
-            loop_variable_symbol = DataSymbol(loop_variable_name, INTEGER_TYPE)
-            symbol_table.add(loop_variable_symbol)
+        loop_variable_symbol = symbol_table.find_or_create(
+                loop_variable_name, symbol_type=DataSymbol,
+                datatype=INTEGER_TYPE)
 
         # Replace the loop_idx array dimension with the loop variable.
         n_ranges = None
-        for array in assignment.walk(ArrayReference):
+        # Just loop the top-level arrays since we just do 1 substitution per
+        # array construct, even if they have nested arrays in turn.
+        for top_level_ref in assignment.walk(ArrayMixin, stop_type=ArrayMixin):
+            # Then start checking with the inner-most array
+            for array in reversed(top_level_ref.walk(ArrayMixin)):
+                current_n_ranges = len([child for child in array.children
+                                        if isinstance(child, Range)])
+                if current_n_ranges == 0:
+                    continue  # This sub-expression already has explicit dims
+                if n_ranges is None:
+                    n_ranges = current_n_ranges
+                elif n_ranges != current_n_ranges:
+                    raise InternalError(
+                        "The number of ranges in the arrays within this "
+                        "assignment are not equal. Any such case should have "
+                        "been dealt with by the validation method or "
+                        "represents invalid PSyIR.")
 
-            # Ignore the array reference if none of its index accesses
-            # are Ranges
-            if not any(child for child in array.children if
-                       isinstance(child, Range)):
-                continue
-            # Ignore the array reference if any of its parents up to
-            # the Assignment node are not Operations that return
-            # scalars.
-            ignore = False
-            current = array.parent
-            while not isinstance(current, Assignment):
-                # Ignore if not a scalar valued operation (vector
-                # valued operations are excluded in the validate
-                # method).
-                if not isinstance(current, Operation):
-                    ignore = True
-                    break
-                current = current.parent
-            if ignore:
-                continue
+                idx = array.get_outer_range_index()
+                array.children[idx] = Reference(loop_variable_symbol)
+                break  # If one is found, go to the next top level expression
 
-            current_n_ranges = len([child for child in array.children
-                                    if isinstance(child, Range)])
-            if n_ranges is None:
-                n_ranges = current_n_ranges
-            elif n_ranges != current_n_ranges:
-                raise InternalError(
-                    "The number of ranges in the arrays within this "
-                    "assignment are not equal. This is invalid PSyIR and "
-                    "should never happen.")
-            idx = get_outer_index(array)
-            array.children[idx] = Reference(loop_variable_symbol)
+        # Replace the assignment with the new explicit loop structure
         position = assignment.position
         loop = NemoLoop.create(loop_variable_symbol, lower_bound,
                                upper_bound, step, [assignment.detach()])
         parent.children.insert(position, loop)
 
-        try:
-            _ = get_outer_index(array_reference)
-        except IndexError:
+        if not assignment.lhs.walk(Range):
             # All valid array ranges have been replaced with explicit
             # loops. We now need to take the content of the loop and
             # place it within a NemoKern (inlined kernel) node.
@@ -276,62 +275,130 @@ class NemoArrayRange2LoopTrans(Transformation):
             Range, if the Range node is not part of an ArrayReference, \
             if the Range node is not the outermost Range node of the \
             ArrayReference or if that ArrayReference does not \
-            consitute the left hand side of an Assignment node.
+            constitute the left hand side of an Assignment node.
+        :raises TransformationError: if the node argument has nested array \
+            expressions with Ranges or is an invalid tree with ranges in \
+            multiple locations of a structure of arrays.
 
         '''
         # Am I Range node?
         if not isinstance(node, Range):
             raise TransformationError(
-                "Error in NemoArrayRange2LoopTrans transformation. The "
-                "supplied node argument should be a PSyIR Range, but "
-                "found '{0}'.".format(type(node).__name__))
+                f"Error in NemoArrayRange2LoopTrans transformation. The "
+                f"supplied node argument should be a PSyIR Range, but "
+                f"found '{type(node).__name__}'.")
         # Am I within an array reference?
-        if not node.parent or not isinstance(node.parent, ArrayReference):
+        if not node.parent or not isinstance(node.parent, ArrayMixin):
             raise TransformationError(
-                "Error in NemoArrayRange2LoopTrans transformation. The "
-                "supplied node argument should be within an ArrayReference "
-                "node, but found '{0}'.".format(type(node.parent).__name__))
-        array_ref = node.parent
+                f"Error in NemoArrayRange2LoopTrans transformation. The "
+                f"supplied node argument should be within an array access "
+                f"node, but found '{type(node.parent).__name__}'.")
         # Is the array reference within an assignment?
-        if not array_ref.parent or not isinstance(array_ref.parent,
-                                                  Assignment):
+        assignment = node.ancestor(Assignment)
+        if not assignment:
             raise TransformationError(
-                "Error in NemoArrayRange2LoopTrans transformation. The "
-                "supplied node argument should be within an ArrayReference "
-                "node that is within an Assignment node, but found '{0}'."
-                .format(type(array_ref.parent).__name__))
-        assignment = array_ref.parent
+                f"Error in NemoArrayRange2LoopTrans transformation. The "
+                f"supplied node argument should be within an Assignment node, "
+                f"but found a '{node}' that is not in an assignment.")
         # Is the array reference the lhs of the assignment?
-        if assignment.lhs is not array_ref:
+        if node not in assignment.lhs.walk(Node):
             raise TransformationError(
                 "Error in NemoArrayRange2LoopTrans transformation. The "
-                "supplied node argument should be within an ArrayReference "
+                "supplied node argument should be within an array access "
                 "node that is within the left-hand-side of an Assignment "
                 "node, but it is on the right-hand-side.")
-        # Does the rhs of the assignment have any operations that
-        # return arrays as this is not currently supported?
+
+        # We don't support nested range expressions
+        for range_expr in assignment.walk(Range):
+            ancestor_array = range_expr.parent.ancestor(ArrayMixin)
+            if ancestor_array and any(index.walk(Range) for index
+                                      in ancestor_array.indices):
+                raise TransformationError(LazyString(
+                    lambda: f"Error in NemoArrayRange2LoopTrans transformation"
+                    f". This transformation does not support array assignments"
+                    f" that contain nested Range structures, but found:"
+                    f"\n{FortranWriter()(assignment)}"))
+
+        # Does the rhs of the assignment have any operations that are not
+        # elemental?
         for operation in assignment.rhs.walk(Operation):
-            # At the moment the only array valued operator is matmul
-            if operation.operator == BinaryOperation.Operator.MATMUL:
+            # Allow non elemental UBOUND and LBOUND
+            if operation.operator is BinaryOperation.Operator.LBOUND:
+                continue
+            if operation.operator is BinaryOperation.Operator.UBOUND:
+                continue
+            if not operation.is_elemental():
                 raise TransformationError(
-                    "Error in NemoArrayRange2LoopTrans transformation. This "
-                    "transformation does not support array valued operations "
-                    "on the rhs of the associated Assignment node, but found "
-                    "'{0}'.".format(operation.operator.name))
+                    f"Error in NemoArrayRange2LoopTrans transformation. This "
+                    f"transformation does not support non-elemental operations"
+                    f" on the rhs of the associated Assignment node, but found"
+                    f" '{operation.operator.name}'.")
+
+        # Do a single walk to avoid doing a separate one for each type we need
+        nodes_to_check = assignment.walk((CodeBlock, Call, Reference))
+
+        # Do not allow to transform expressions with CodeBlocks
+        if any(isinstance(n, CodeBlock) for n in nodes_to_check):
+            raise TransformationError(LazyString(
+                lambda: f"Error in NemoArrayRange2LoopTrans transformation. "
+                f"This transformation does not support array assignments that"
+                f" contain a CodeBlock anywhere in the expression, but found:"
+                f"\n{FortranWriter()(assignment)}"))
+        # Do not allow to transform expressions with function calls (to allow
+        # this we need to differentiate between elemental and not elemental
+        # functions as they have different semantics in array notation)
+        if any(isinstance(n, Call) for n in nodes_to_check):
+            raise TransformationError(LazyString(
+                lambda: f"Error in NemoArrayRange2LoopTrans transformation. "
+                f"This transformation does not support array assignments that"
+                f" contain a Call anywhere in the expression, but found:"
+                f"\n{FortranWriter()(assignment)}"))
+
+        references = [n for n in nodes_to_check if isinstance(n, Reference)]
+        for reference in references:
+            # As special case we always allow references to whole arrays as
+            # part of the LBOUND and UBOUND operations, regardless of the
+            # restrictions below (e.g. is a DeferredType reference).
+            if isinstance(reference.parent, Operation):
+                operator = reference.parent.operator
+                if operator is BinaryOperation.Operator.LBOUND:
+                    continue
+                if operator is BinaryOperation.Operator.UBOUND:
+                    continue
+
+            # We allow any references that are part of a structure syntax - we
+            # analyse its child components by continuing the reference list
+            if isinstance(reference, (StructureReference, StructureMember)):
+                continue
+
+            # We allow any references that have explicit array syntax
+            # because we infer that they are not scalars from the context
+            # where they are found (even if they have DeferredType)
+            if isinstance(reference, (ArrayReference, ArrayMember)):
+                continue
+
+            # However, if it doesn't have array accessors or structure syntax,
+            # we must be sure that it represents a scalar.
+            if not isinstance(reference.symbol, DataSymbol) or \
+                    not isinstance(reference.symbol.datatype, ScalarType):
+                raise TransformationError(
+                    f"Error in NemoArrayRange2LoopTrans transformation. "
+                    f"Variable '{reference.symbol.name}' must be a DataSymbol"
+                    f" of ScalarType, but it's a '{reference.symbol}'.")
 
         # Is the Range node the outermost Range (as if not, the
         # transformation would be invalid)?
-        if any(isinstance(node, Range) for node in
-               node.parent.children[node.position+1:]):
-            raise TransformationError(
-                "Error in NemoArrayRange2LoopTrans transformation. This "
-                "transformation can only be applied to the outermost "
-                "Range.")
+        for child in node.parent.indices[node.parent.indices.index(node)+1:]:
+            if isinstance(child, Range):
+                raise TransformationError(
+                    "Error in NemoArrayRange2LoopTrans transformation. This "
+                    "transformation can only be applied to the outermost "
+                    "Range.")
 
         # If there is a loop variable defined in the config file and
         # this variable is already defined in the code, is it defined
         # as an integer?
-        array_index = node.position
+        array_index = node.parent.indices.index(node)
         loop_type_order = Config.get().api_conf("nemo").get_index_order()
         loop_type_data = Config.get().api_conf("nemo").get_loop_type_data()
         try:
@@ -347,36 +414,40 @@ class NemoArrayRange2LoopTrans(Transformation):
                         loop_variable_symbol.datatype.intrinsic is
                         ScalarType.Intrinsic.INTEGER):
                     raise TransformationError(
-                        "The config file specifies '{0}' as the name of the "
-                        "iteration variable but this is already declared in "
-                        "the code as something that is not a scalar integer, "
-                        "or is a deferred type.".format(loop_variable_name))
+                        f"The config file specifies '{loop_variable_name}' as "
+                        f"the name of the iteration variable but this is "
+                        f"already declared in the code as something that is "
+                        f"not a scalar integer or a deferred type.")
             except KeyError:
                 # Variable is not defined
                 pass
+
+            # Make sure this array is not already inside a loop with the
+            # loop_variable_name that we choose. If it is we ignore it (as
+            # usually we will already have some explicit loops above this
+            # construct to apply further parallelisation/optimisations).
+            current = node.parent
+            while not isinstance(current, Routine):
+                if isinstance(current, Loop):
+                    varname = current.variable.name
+                    if varname.lower() == loop_variable_name.lower():
+                        raise TransformationError(LazyString(
+                            lambda: f"The config file specifies "
+                            f"'{loop_variable_name}' as the name of the "
+                            f"iteration variable but this is already used "
+                            f"by an ancestor loop variable in:"
+                            f"\n{FortranWriter()(current)}"))
+                current = current.parent
+            # Note that we don't check if the variable name is used in a
+            # context other that the loop if it's an integer. We assume this
+            # is a guarantee of the API.
+
         except IndexError:
-            # There is no name for this index in the config file
+            # There is no name for this index in the config file (it can
+            # proceed as the apply will generate a new variable for it)
             pass
 
 
-def get_outer_index(array):
-    '''Find the outermost index of the array that is a Range node. If one
-    does not exist then raise an exception.
-
-    :param array: the array being examined.
-    :type array: :py:class:`psyclone.psyir.nodes.ArrayReference`
-
-    :returns: the outermost index of the array that is a Range node.
-
-    :raises IndexError: if the array does not contain a Range node.
-
-    '''
-    for child in reversed(array.children):
-        if isinstance(child, Range):
-            return child.position
-    raise IndexError
-
-
 # For automatic document generation
 __all__ = [
     'NemoArrayRange2LoopTrans']
