@@ -44,6 +44,7 @@ from __future__ import print_function, absolute_import
 from fparser.two.utils import walk
 from fparser.two import Fortran2003
 from psyclone.configuration import Config
+from psyclone.domain.nemo import NemoConstants
 from psyclone.psyGen import PSy, Invokes, Invoke, InvokeSchedule, \
     InlinedKern
 from psyclone.errors import InternalError
@@ -93,8 +94,8 @@ class NemoInvokes(Invokes):
         # Transform the language-level PSyIR into NEMO-specific PSyIR
         # pylint: disable=import-outside-toplevel
         from psyclone.domain.nemo.transformations import CreateNemoPSyTrans
-        container, _ = CreateNemoPSyTrans().apply(psyir)
-        routines = container.walk(Routine)
+        CreateNemoPSyTrans().apply(psyir)
+        routines = psyir.walk(Routine)
 
         # Create an Invoke for each routine we've found
         for subroutine in routines:
@@ -255,18 +256,13 @@ class NemoLoop(Loop):
     :type parent: :py:class:`psyclone.psyir.nodes.Node`
     :param str variable_name: optional name of the loop iterator \
         variable. Defaults to an empty string.
+
     '''
     def __init__(self, parent=None, variable=None):
-        # The order in which names are returned in
-        # get_valid_loop_types depends on the Python implementation
-        # (as it pulls the values from a dictionary). To make it clear
-        # that there is no implied ordering here we store
-        # valid_loop_types as a set, rather than a list.
-        valid_loop_types = set(
-            Config.get().api_conf("nemo").get_valid_loop_types())
+        const = NemoConstants()
         Loop.__init__(self, parent=parent,
                       variable=variable,
-                      valid_loop_types=valid_loop_types)
+                      valid_loop_types=const.VALID_LOOP_TYPES)
 
     @staticmethod
     def create(variable, start, stop, step, children):
