@@ -1,7 +1,7 @@
 # -----------------------------------------------------------------------------
 # BSD 3-Clause License
 #
-# Copyright (c) 2017-2021, Science and Technology Facilities Council.
+# Copyright (c) 2017-2022, Science and Technology Facilities Council.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -40,7 +40,6 @@
 
 from psyclone.psyir.nodes.datanode import DataNode
 from psyclone.psyir.nodes.statement import Statement
-from psyclone.psyir.nodes.node import colored
 from psyclone.psyir.nodes.routine import Routine
 from psyclone.psyir.nodes import Schedule, Literal
 from psyclone.psyir.symbols import ScalarType, DataSymbol
@@ -50,6 +49,7 @@ from psyclone.f2pygen import DoGen, DeclGen
 
 
 class Loop(Statement):
+    # pylint: disable=too-many-instance-attributes
     '''Node representing a loop within the PSyIR. It has 4 mandatory children:
     the first one represents the loop lower bound, the second one represents
     the loop upper bound, the third one represents the step value and the
@@ -78,7 +78,7 @@ class Loop(Statement):
                            without the 'was_where' annotation.
 
     '''
-    valid_annotations = ('was_where', 'was_single_stmt')
+    valid_annotations = ('was_where', 'was_single_stmt', 'chunked')
     # Textual description of the node.
     _children_valid_format = "DataNode, DataNode, DataNode, Schedule"
     _text_name = "Loop"
@@ -308,6 +308,14 @@ class Loop(Statement):
             name = "loop_" + str(position)
         return name
 
+    @property
+    def valid_loop_types(self):
+        '''
+        :returns: the (domain-specific) loop types allowed by this instance.
+        :rtype: list of str
+        '''
+        return self._valid_loop_types
+
     @property
     def loop_type(self):
         '''
@@ -342,7 +350,7 @@ class Loop(Statement):
         :rtype: str
         '''
         return ("{0}[type='{1}', field_space='{2}', it_space='{3}']".
-                format(colored("Loop", self._colour),
+                format(self.coloured_name(self._colour),
                        self._loop_type, self._field_space,
                        self.iteration_space))
 
@@ -555,37 +563,29 @@ class Loop(Statement):
             calls = self.reductions()
             zero_reduction_variables(calls, parent)
 
-        invoke = self.ancestor(InvokeSchedule)
-        if (invoke and invoke.opencl) or (
-                is_unit_literal(self.start_expr) and
-                is_unit_literal(self.stop_expr)):
-            # no need for a loop
-            for child in self.loop_body:
-                child.gen_code(parent)
+        # Avoid circular dependency
+        # pylint: disable=import-outside-toplevel
+        from psyclone.psyir.backend.fortran import FortranWriter
+        # start/stop/step_expr are generated with the FortranWriter
+        # backend, the rest of the loop with f2pygen.
+        fwriter = FortranWriter()
+        if is_unit_literal(self.step_expr):
+            step_str = None
         else:
-            # Avoid circular dependency
-            # pylint: disable=import-outside-toplevel
-            from psyclone.psyir.backend.fortran import FortranWriter
-            # start/stop/step_expr are generated with the FortranWriter
-            # backend, the rest of the loop with f2pygen.
-            fwriter = FortranWriter()
-            if is_unit_literal(self.step_expr):
-                step_str = None
-            else:
-                step_str = fwriter(self.step_expr)
+            step_str = fwriter(self.step_expr)
 
-            do_stmt = DoGen(parent, self.variable.name,
-                            fwriter(self.start_expr),
-                            fwriter(self.stop_expr),
-                            step_str)
-            # need to add do loop before children as children may want to add
-            # info outside of do loop
-            parent.add(do_stmt)
-            for child in self.loop_body:
-                child.gen_code(do_stmt)
-            my_decl = DeclGen(parent, datatype="integer",
-                              entity_decls=[self.variable.name])
-            parent.add(my_decl)
+        do_stmt = DoGen(parent, self.variable.name,
+                        fwriter(self.start_expr),
+                        fwriter(self.stop_expr),
+                        step_str)
+        # need to add do loop before children as children may want to add
+        # info outside of do loop
+        parent.add(do_stmt)
+        for child in self.loop_body:
+            child.gen_code(do_stmt)
+        my_decl = DeclGen(parent, datatype="integer",
+                          entity_decls=[self.variable.name])
+        parent.add(my_decl)
 
     def _halo_read_access(self, arg):
         '''Determines whether the supplied argument has (or might have) its
