@@ -1,7 +1,7 @@
 # -----------------------------------------------------------------------------
 # BSD 3-Clause License
 #
-# Copyright (c) 2020-2021, Science and Technology Facilities Council.
+# Copyright (c) 2020-2022, Science and Technology Facilities Council.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -75,10 +75,9 @@ class Member(Node):
         if (parent and
                 not isinstance(parent, (StructureReference, StructureMember))):
             raise TypeError(
-                "The parent of a {0} must be either a "
-                "(ArrayOf)Structure(s)Reference or (ArrayOf)Structure(s)Member"
-                " but found '{1}'.".format(type(self).__name__,
-                                           type(parent).__name__))
+                f"The parent of a {type(self).__name__} must be either a "
+                f"(ArrayOf)Structure(s)Reference or (ArrayOf)Structure(s)"
+                f"Member but found '{type(parent).__name__}'.")
 
         super(Member, self).__init__(parent=parent)
         # Store the name of the component that this member represents
@@ -106,6 +105,15 @@ class Member(Node):
     def __str__(self):
         return self.node_str(False)
 
+    @property
+    def is_array(self):
+        ''':returns: if this member is an array.
+        :rtype: bool
+
+        '''
+        # pylint: disable=no-self-use
+        return False
+
     def get_signature_and_indices(self):
         ''':returns: the Signature of this member access, and a list \
         of list of the indices used for each component, which is empty \
