@@ -1,7 +1,7 @@
 # -----------------------------------------------------------------------------
 # BSD 3-Clause License
 #
-# Copyright (c) 2017-2021, Science and Technology Facilities Council.
+# Copyright (c) 2017-2022, Science and Technology Facilities Council.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -321,6 +321,7 @@ class Node(object):
         ignoring these tags.
     :type annotations: list of str
 
+    :raises TypeError: if a parent is supplied that is not an instance of Node.
     :raises InternalError: if an invalid annotation tag is supplied.
 
     '''
@@ -347,6 +348,9 @@ class Node(object):
                                       self._children_valid_format)
         if children:
             self._children.extend(children)
+        if parent and not isinstance(parent, Node):
+            raise TypeError(f"The parent of a Node must also be a Node but "
+                            f"got '{type(parent).__name__}'")
         # Keep a record of whether a parent node was supplied when constructing
         # this object. In this case it still won't appear in the parent's
         # children list. When both ends of the reference are connected this
@@ -439,30 +443,6 @@ class Node(object):
     def __str__(self):
         return self.node_str(False)
 
-    def math_equal(self, other):
-        '''Returns True if the self has the same results as other. The
-        implementation in the base class just confirms that the type is the
-        same, and the number of children as well.
-
-        :param other: the node to compare self with.
-        :type other: py:class:`psyclone.psyir.nodes.Node`.
-
-        :returns: whether self has the same result as other.
-        :rtype: bool
-        '''
-
-        # pylint: disable=unidiomatic-typecheck
-        if type(self) != type(other):
-            return False
-
-        if len(self.children) != len(other.children):
-            return False
-
-        for i, entity in enumerate(self.children):
-            if not entity.math_equal(other.children[i]):
-                return False
-        return True
-
     @property
     def ast(self):
         '''
