@@ -1,7 +1,7 @@
 # -----------------------------------------------------------------------------
 # BSD 3-Clause License
 #
-# Copyright (c) 2021, Science and Technology Facilities Council.
+# Copyright (c) 2021-2022, Science and Technology Facilities Council.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -54,14 +54,19 @@ from psyclone.f2pygen import (AssignGen, UseGen, DeclGen, DirectiveGen,
 from psyclone.psyir.nodes.directive import StandaloneDirective, \
     RegionDirective
 from psyclone.psyir.nodes.loop import Loop
+from psyclone.psyir.nodes.literal import Literal
+from psyclone.psyir.nodes.routine import Routine
+from psyclone.psyir.nodes.omp_clauses import OMPGrainsizeClause, \
+    OMPNowaitClause, OMPNogroupClause, OMPNumTasksClause
+from psyclone.psyir.nodes.schedule import Schedule
+from psyclone.psyir.symbols import INTEGER_TYPE
 
 # OMP_OPERATOR_MAPPING is used to determine the operator to use in the
 # reduction clause of an OpenMP directive.
 OMP_OPERATOR_MAPPING = {AccessType.SUM: "+"}
 
 
-@six.add_metaclass(abc.ABCMeta)
-class OMPDirective():
+class OMPDirective(metaclass=abc.ABCMeta):
     '''
     Base mixin class for all OpenMP-related directives.
 
@@ -116,6 +121,56 @@ class OMPStandaloneDirective(OMPDirective, StandaloneDirective):
     '''
 
 
+class OMPDeclareTargetDirective(OMPStandaloneDirective):
+    '''
+    Class representing an OpenMP Declare Target directive in the PSyIR.
+
+    '''
+    def gen_code(self, parent):
+        '''Generate the fortran OMP Declare Target Directive and any
+        associated code.
+
+        :param parent: the parent Node in the Schedule to which to add our \
+                       content.
+        :type parent: sub-class of :py:class:`psyclone.f2pygen.BaseGen`
+        '''
+        # Check the constraints are correct
+        self.validate_global_constraints()
+
+        # Generate the code for this Directive
+        parent.add(DirectiveGen(parent, "omp", "begin", "declare", "target"))
+
+    def begin_string(self):
+        '''Returns the beginning statement of this directive, i.e.
+        "omp routine". The visitor is responsible for adding the
+        correct directive beginning (e.g. "!$").
+
+        :returns: the opening statement of this directive.
+        :rtype: str
+
+        '''
+        # pylint: disable=no-self-use
+        return "omp declare target"
+
+    def validate_global_constraints(self):
+        '''
+        Perform validation checks that can only be done at code-generation
+        time.
+
+        :raises GenerationError: if this directive is not the first statement \
+            in a routine.
+
+        '''
+        if self.parent and (not isinstance(self.parent, Routine) or
+                            self.parent.children[0] is not self):
+            raise GenerationError(
+                f"A OMPDeclareTargetDirective must be the first child (index "
+                f"0) of a Routine but found one as child {self.position} of a "
+                f"{type(self.parent).__name__}.")
+
+        super().validate_global_constraints()
+
+
 class OMPTaskwaitDirective(OMPStandaloneDirective):
     '''
     Class representing an OpenMP TASKWAIT directive in the PSyIR.
@@ -226,6 +281,7 @@ class OMPSingleDirective(OMPSerialDirective):
         a nowait clause applied. Default value is False.
 
     '''
+    _children_valid_format = "Schedule, [OMPNowaitClause]"
     # Textual description of the node
     _text_name = "OMPSingleDirective"
 
@@ -236,6 +292,30 @@ class OMPSingleDirective(OMPSerialDirective):
         # the nowait requirement
         super(OMPSingleDirective, self).__init__(children=children,
                                                  parent=parent)
+        if self._nowait:
+            self.children.append(OMPNowaitClause())
+
+    @staticmethod
+    def _validate_child(position, child):
+        '''
+         Decides whether a given child and position are valid for this node.
+         The rules are:
+         1. Child 0 must always be a Schedule.
+         2. Child 1 can only be a OMPNowaitClause.
+
+        :param int position: the position to be validated.
+        :param child: a child to be validated.
+        :type child: :py:class:`psyclone.psyir.nodes.Node`
+
+        :return: whether the given child and position are valid for this node.
+        :rtype: bool
+
+        '''
+        if position == 0:
+            return isinstance(child, Schedule)
+        if position == 1:
+            return isinstance(child, OMPNowaitClause)
+        return False
 
     @property
     def nowait(self):
@@ -266,7 +346,7 @@ class OMPSingleDirective(OMPSerialDirective):
                                 nowait_string))
 
         # Generate the code for all of this node's children
-        for child in self.children:
+        for child in self.dir_body:
             child.gen_code(parent)
 
         # Generate the end code for this node
@@ -281,11 +361,8 @@ class OMPSingleDirective(OMPSerialDirective):
         :rtype: str
 
         '''
-        result = "omp single"
-
-        if self._nowait:
-            result = result + " nowait"
-        return result
+        # pylint: disable=no-self-use
+        return "omp single"
 
     def end_string(self):
         '''Returns the end (or closing) statement of this directive, i.e.
@@ -587,9 +664,17 @@ class OMPTaskloopDirective(OMPRegionDirective):
                              a grainsize and num_tasks value \
                              specified.
     '''
+    # This specification respects the mutual exclusion of OMPGransizeClause
+    # and OMPNumTasksClause, but adds an additional ordering requirement.
+    # Other specifications to soften the ordering requirement are possible,
+    # but need additional checks in the global constraints instead.
+    _children_valid_format = ("Schedule, [OMPGrainsizeClause | "
+                              "OMPNumTasksClause], [OMPNogroupClause]")
+
     # pylint: disable=too-many-arguments
     def __init__(self, children=None, parent=None, grainsize=None,
                  num_tasks=None, nogroup=False):
+        # These remain primarily for the gen_code interface
         self._grainsize = grainsize
         self._num_tasks = num_tasks
         self._nogroup = nogroup
@@ -599,6 +684,43 @@ class OMPTaskloopDirective(OMPRegionDirective):
                 "numtasks clauses specified.")
         super(OMPTaskloopDirective, self).__init__(children=children,
                                                    parent=parent)
+        if self._grainsize is not None:
+            child = [Literal(f"{grainsize}", INTEGER_TYPE)]
+            self._children.append(OMPGrainsizeClause(children=child))
+        if self._num_tasks is not None:
+            child = [Literal(f"{num_tasks}", INTEGER_TYPE)]
+            self._children.append(OMPNumTasksClause(children=child))
+        if self._nogroup:
+            self._children.append(OMPNogroupClause())
+
+    @staticmethod
+    def _validate_child(position, child):
+        '''
+         Decides whether a given child and position are valid for this node.
+         The rules are:
+         1. Child 0 must always be a Schedule.
+         2. Child 1 may be either a OMPGrainsizeClause or OMPNumTasksClause, \
+            or if neither of those clauses are present, it may be a \
+            OMPNogroupClause.
+         3. Child 2 must always be a OMPNogroupClause, and can only exist if \
+            child 1 is a OMPGrainsizeClause or OMPNumTasksClause.
+
+        :param int position: the position to be validated.
+        :param child: a child to be validated.
+        :type child: :py:class:`psyclone.psyir.nodes.Node`
+
+        :return: whether the given child and position are valid for this node.
+        :rtype: bool
+
+        '''
+        if position == 0:
+            return isinstance(child, Schedule)
+        if position == 1:
+            return isinstance(child, (OMPGrainsizeClause, OMPNumTasksClause,
+                                      OMPNogroupClause))
+        if position == 2:
+            return isinstance(child, OMPNogroupClause)
+        return False
 
     @property
     def nogroup(self):
@@ -615,6 +737,8 @@ class OMPTaskloopDirective(OMPRegionDirective):
 
         :raises GenerationError: if this OMPTaskloopDirective is not \
                                  enclosed within an OpenMP serial region.
+        :raises GenerationError: if this OMPTaskloopDirective has two
+                                 Nogroup clauses as children.
         '''
         # It is only at the point of code generation that we can check for
         # correctness (given that we don't mandate the order that a user
@@ -626,6 +750,14 @@ class OMPTaskloopDirective(OMPRegionDirective):
                 "OMPTaskloopDirective must be inside an OMP Serial region "
                 "but could not find an ancestor node")
 
+        # Check children are well formed.
+        # _validate_child will ensure position 0 and 1 are valid.
+        if len(self._children) == 3 and isinstance(self._children[1],
+                                                   OMPNogroupClause):
+            raise GenerationError(
+                "OMPTaskloopDirective has two Nogroup clauses as children "
+                "which is not allowed.")
+
         super(OMPTaskloopDirective, self).validate_global_constraints()
 
     def gen_code(self, parent):
@@ -652,15 +784,13 @@ class OMPTaskloopDirective(OMPRegionDirective):
             clause_list.append("num_tasks({0})".format(self._num_tasks))
         if self._nogroup:
             clause_list.append("nogroup")
-
         # Generate the string containing the required clauses
         extra_clauses = ", ".join(clause_list)
 
         parent.add(DirectiveGen(parent, "omp", "begin", "taskloop",
                                 extra_clauses))
 
-        for child in self.children:
-            child.gen_code(parent)
+        self.dir_body.gen_code(parent)
 
         # make sure the directive occurs straight after the loop body
         position = parent.previous_loop()
@@ -669,31 +799,19 @@ class OMPTaskloopDirective(OMPRegionDirective):
 
     def begin_string(self):
         '''Returns the beginning statement of this directive, i.e.
-        "omp do ...". The visitor is responsible for adding the
+        "omp taskloop ...". The visitor is responsible for adding the
         correct directive beginning (e.g. "!$").
 
         :returns: the beginning statement for this directive.
         :rtype: str
 
         '''
-        extra_clauses = ""
-        # Find the specified clauses
-        clause_list = []
-        if self._grainsize is not None:
-            clause_list.append(" grainsize({0})".format(self._grainsize))
-        if self._num_tasks is not None:
-            clause_list.append(" num_tasks({0})".format(self._num_tasks))
-        if self._nogroup:
-            clause_list.append(" nogroup")
-
-        # Generate the string containing the required clauses
-        extra_clauses = ",".join(clause_list)
-
-        return "omp taskloop" + extra_clauses
+        # pylint: disable=no-self-use
+        return "omp taskloop"
 
     def end_string(self):
         '''Returns the end (or closing) statement of this directive, i.e.
-        "omp end do". The visitor is responsible for adding the
+        "omp end taskloop". The visitor is responsible for adding the
         correct directive beginning (e.g. "!$").
 
         :returns: the end statement for this directive.
@@ -948,6 +1066,18 @@ class OMPTargetDirective(OMPRegionDirective):
         # pylint: disable=no-self-use
         return "omp target"
 
+    def end_string(self):
+        '''Returns the end (or closing) statement of this directive, i.e.
+        "omp end target". The visitor is responsible for adding the
+        correct directive beginning (e.g. "!$").
+
+        :returns: the end statement for this directive.
+        :rtype: str
+
+        '''
+        # pylint: disable=no-self-use
+        return "omp end target"
+
 
 class OMPLoopDirective(OMPRegionDirective):
     ''' Class for the !$OMP LOOP directive that specifies that the iterations
@@ -1026,25 +1156,32 @@ class OMPLoopDirective(OMPRegionDirective):
             string += " collapse({0})".format(str(self._collapse))
         return string
 
+    def end_string(self):
+        '''Returns the end (or closing) statement of this directive, i.e.
+        "omp end loop". The visitor is responsible for adding the
+        correct directive beginning (e.g. "!$").
+
+        :returns: the end statement for this directive.
+        :rtype: str
+
+        '''
+        # pylint: disable=no-self-use
+        return "omp end loop"
+
     def validate_global_constraints(self):
         ''' Perform validation of those global constraints that can only be
         done at code-generation time.
 
-        :raises GenerationError: if this OMPLoopDirective is not enclosed \
-            within some OMPParallelDirective region.
         :raises GenerationError: if this OMPLoopDirective has more than one \
             child in its associated schedule.
         :raises GenerationError: if the schedule associated with this \
             OMPLoopDirective does not contain a Loop.
+        :raises GenerationError: this directive must be inside a omp target \
+            or parallel region.
         :raises GenerationError: if this OMPLoopDirective has a collapse \
             clause but it doesn't have the expected number of nested Loops.
 
         '''
-        if not self.ancestor(OMPParallelDirective):
-            raise GenerationError(
-                "OMPLoopDirective must have an OMPParallelDirective as an "
-                "ancestor.")
-
         if len(self.dir_body.children) != 1:
             raise GenerationError(
                 "OMPLoopDirective must have exactly one child in its "
@@ -1056,6 +1193,13 @@ class OMPLoopDirective(OMPRegionDirective):
                 "OMPLoopDirective must have a Loop as child of its associated "
                 "schedule but found '{0}'.".format(self.dir_body.children[0]))
 
+        if not self.ancestor((OMPTargetDirective, OMPParallelDirective)):
+            # Also omp teams or omp threads regions but these are not supported
+            # in the PSyIR
+            raise GenerationError(
+                f"OMPLoopDirective must be inside a OMPTargetDirective or a "
+                f"OMPParallelDirective, but '{self}' is not.")
+
         # If there is a collapse clause, there must be as many immediately
         # nested loops as the collapse value
         if self._collapse:
@@ -1079,4 +1223,4 @@ __all__ = ["OMPRegionDirective", "OMPParallelDirective", "OMPSingleDirective",
            "OMPMasterDirective", "OMPDoDirective", "OMPParallelDoDirective",
            "OMPSerialDirective", "OMPTaskloopDirective", "OMPTargetDirective",
            "OMPTaskwaitDirective", "OMPDirective", "OMPStandaloneDirective",
-           "OMPLoopDirective"]
+           "OMPLoopDirective", "OMPDeclareTargetDirective"]
