@@ -1,7 +1,7 @@
 # -----------------------------------------------------------------------------
 # BSD 3-Clause License
 #
-# Copyright (c) 2017-2021, Science and Technology Facilities Council.
+# Copyright (c) 2017-2022, Science and Technology Facilities Council.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -42,8 +42,9 @@ sub-classes.'''
 import abc
 from enum import Enum
 import six
-from psyclone.psyir.nodes.datanode import DataNode
+
 from psyclone.errors import GenerationError
+from psyclone.psyir.nodes.datanode import DataNode
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -65,6 +66,7 @@ class Operation(DataNode):
     # Must be overridden in sub-class to hold an Enumeration of the Operators
     # that it can represent.
     Operator = object
+    _non_elemental_ops = []
     # Textual description of the node.
     _text_name = "Operation"
     _colour = "blue"
@@ -74,9 +76,9 @@ class Operation(DataNode):
 
         if not isinstance(operator, self.Operator):
             raise TypeError(
-                "{0} operator argument must be of type "
-                "{0}.Operator but found {1}.".format(type(self).__name__,
-                                                     type(operator).__name__))
+                f"{type(self).__name__} operator argument must be of type "
+                f"{type(self).__name__}.Operator but found "
+                f"{type(operator).__name__}.")
         self._operator = operator
 
     @property
@@ -105,6 +107,15 @@ class Operation(DataNode):
         return self.coloured_name(colour) + \
             "[operator:'" + self._operator.name + "']"
 
+    def is_elemental(self):
+        '''
+        :returns: whether this operation is elemental (provided with an input \
+            array it will apply the operation individually to each of the \
+            array elements and return an array with the results).
+        :rtype: bool
+        '''
+        return self.operator not in self._non_elemental_ops
+
     def __str__(self):
         result = self.node_str(False) + "\n"
         for entity in self._children:
@@ -138,6 +149,8 @@ class UnaryOperation(Operation):
         'REAL', 'INT', 'NINT'
         ])
 
+    _non_elemental_ops = [Operator.SUM]
+
     @staticmethod
     def _validate_child(position, child):
         '''
@@ -169,9 +182,9 @@ class UnaryOperation(Operation):
         '''
         if not isinstance(oper, UnaryOperation.Operator):
             raise GenerationError(
-                "oper argument in create method of UnaryOperation class "
-                "should be a PSyIR UnaryOperation Operator but found '{0}'."
-                "".format(type(oper).__name__))
+                f"oper argument in create method of UnaryOperation class "
+                f"should be a PSyIR UnaryOperation Operator but found "
+                f"'{type(oper).__name__}'.")
 
         unary_op = UnaryOperation(oper)
         unary_op.children = [child]
@@ -198,8 +211,11 @@ class BinaryOperation(Operation):
         # Array Query Operators
         'SIZE', 'LBOUND', 'UBOUND',
         # Matrix and Vector Operators
-        'MATMUL'
+        'MATMUL', 'DOT_PRODUCT'
         ])
+    _non_elemental_ops = [Operator.SUM, Operator.MATMUL, Operator.SIZE,
+                          Operator.LBOUND, Operator.UBOUND,
+                          Operator.DOT_PRODUCT]
     '''Arithmetic operators:
 
     .. function:: POW(arg0, arg1) -> type(arg0)
@@ -262,6 +278,15 @@ class BinaryOperation(Operation):
         same and the resultant data will also have the same
         type. Currently only REAL data is supported.
 
+    .. function:: DOT_PRODUCT(vector1, vector2) -> scalar
+
+       :returns: the result of performing a dot product on two equal \
+           sized vectors.
+
+    .. note:: The type of data in `vector1` and `vector2` must be the
+        same and the resultant data will also have the same
+        type. Currently only REAL data is supported.
+
     '''
     # Textual description of the node.
     _children_valid_format = "DataNode, DataNode"
@@ -280,30 +305,6 @@ class BinaryOperation(Operation):
         '''
         return position in (0, 1) and isinstance(child, DataNode)
 
-    def math_equal(self, other):
-        ''':param other: the node to compare self with.
-        :type other: py:class:`psyclone.psyir.nodes.Node`
-        :returns: True if the self has the same results as other.
-        :rtype: bool
-        '''
-        if not super(BinaryOperation, self).math_equal(other):
-            # Support some commutative law, unfortunately we now need
-            # to repeat some tests already done in super(), since we
-            # don't know why the above test failed
-            # TODO #533 for documenting restrictions
-            # pylint: disable=unidiomatic-typecheck
-            if type(self) != type(other):
-                return False
-            if self.operator != other.operator:
-                return False
-            if self.operator not in [self.Operator.ADD, self.Operator.MUL,
-                                     self.Operator.AND, self.Operator.OR,
-                                     self.Operator.EQ]:
-                return False
-            return self._children[0].math_equal(other.children[1]) and \
-                self._children[1].math_equal(other.children[0])
-        return self.operator == other.operator
-
     @staticmethod
     def create(oper, lhs, rhs):
         '''Create a BinaryOperator instance given an operator and lhs and rhs
@@ -328,9 +329,9 @@ class BinaryOperation(Operation):
         '''
         if not isinstance(oper, BinaryOperation.Operator):
             raise GenerationError(
-                "oper argument in create method of BinaryOperation class "
-                "should be a PSyIR BinaryOperation Operator but found '{0}'."
-                "".format(type(oper).__name__))
+                f"oper argument in create method of BinaryOperation class "
+                f"should be a PSyIR BinaryOperation Operator but found "
+                f"'{type(oper).__name__}'.")
 
         binary_op = BinaryOperation(oper)
         binary_op.children = [lhs, rhs]
@@ -351,6 +352,7 @@ class NaryOperation(Operation):
         # Arithmetic Operators
         'MAX', 'MIN', 'SUM'
         ])
+    _non_elemental_ops = [Operator.SUM]
 
     @staticmethod
     def _validate_child(position, child):
@@ -386,14 +388,13 @@ class NaryOperation(Operation):
         '''
         if not isinstance(oper, NaryOperation.Operator):
             raise GenerationError(
-                "oper argument in create method of NaryOperation class "
-                "should be a PSyIR NaryOperation Operator but found '{0}'."
-                "".format(type(oper).__name__))
+                f"oper argument in create method of NaryOperation class "
+                f"should be a PSyIR NaryOperation Operator but found "
+                f"'{type(oper).__name__}'.")
         if not isinstance(children, list):
             raise GenerationError(
-                "children argument in create method of NaryOperation class "
-                "should be a list but found '{0}'."
-                "".format(type(children).__name__))
+                f"children argument in create method of NaryOperation class "
+                f"should be a list but found '{type(children).__name__}'.")
 
         nary_op = NaryOperation(oper)
         nary_op.children = children
