@@ -1,7 +1,7 @@
 # -----------------------------------------------------------------------------
 # BSD 3-Clause License
 #
-# Copyright (c) 2017-2021, Science and Technology Facilities Council.
+# Copyright (c) 2017-2022, Science and Technology Facilities Council.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -62,6 +62,13 @@ class SymbolError(PSycloneError):
 class SymbolInterface(object):   # pylint: disable=too-few-public-methods
     ''' Abstract class of a Symbol Interface '''
 
+    def copy(self):
+        '''
+        :returns: a copy of this object.
+        :rtype: :py:class:`psyclone.psyir.symbol.SymbolInterface`
+        '''
+        return self.__class__()
+
 
 class LocalInterface(SymbolInterface):
     # pylint: disable=too-few-public-methods
@@ -117,6 +124,13 @@ class ImportInterface(SymbolInterface):
     def __str__(self):
         return "Import(container='{0}')".format(self.container_symbol.name)
 
+    def copy(self):
+        '''
+        :returns: a copy of this object.
+        :rtype: :py:class:`psyclone.psyir.symbol.SymbolInterface`
+        '''
+        return self.__class__(self.container_symbol)
+
 
 class ArgumentInterface(SymbolInterface):
     '''
@@ -181,6 +195,13 @@ class ArgumentInterface(SymbolInterface):
     def __str__(self):
         return "Argument(pass-by-value={0})".format(self._pass_by_value)
 
+    def copy(self):
+        '''
+        :returns: a copy of this object.
+        :rtype: :py:class:`psyclone.psyir.symbol.SymbolInterface`
+        '''
+        return self.__class__(access=self.access)
+
 
 class Symbol(object):
     '''Generic Symbol item for the Symbol Table and PSyIR References.
@@ -270,7 +291,7 @@ class Symbol(object):
         # The constructors for all Symbol-based classes have 'name' as the
         # first positional argument.
         return type(self)(self.name, visibility=self.visibility,
-                          interface=self.interface)
+                          interface=self.interface.copy())
 
     def copy_properties(self, symbol_in):
         '''Replace all properties in this object with the properties from
@@ -561,8 +582,6 @@ class Symbol(object):
         :returns: if the variable is an array.
         :rtype bool:
 
-        :raises InternalError: if a loop_variable is specified, but no \
-            access information is given.
         :raises InternalError: if a loop_variable is specified, but no \
             access information is given.
 
@@ -608,4 +627,10 @@ class Symbol(object):
         # is just a Symbol, not a DataSymbol), the `is_array` function will
         # raise an exception.
         # TODO #1213: check for wildcard imports
-        return self.is_array
+        try:
+            return self.is_array
+        except ValueError:
+            # Generic symbols produce a ValueError, since this does not have
+            # a datatype and an Array access was not found, we don't consider
+            # it an array.
+            return False
