@@ -67,7 +67,8 @@ class OMPTaskwaitTrans(Transformation):
     >>> psy = PSyFactory(api).create(invokeInfo)
     >>>
     >>> from psyclone.transformations import OMPParallelTrans, OMPSingleTrans
-    >>> from psyclone.transformations import OMPTaskloopTrans, OMPTaskwaitTrans
+    >>> from psyclone.transformations import OMPTaskloopTrans
+    >>> from psyclone.psyir.transformations import OMPTaskwaitTrans
     >>> singletrans = OMPSingleTrans()
     >>> paralleltrans = OMPParallelTrans()
     >>> tasklooptrans = OMPTaskloopTrans()
@@ -396,8 +397,6 @@ class OMPTaskwaitTrans(Transformation):
                 result of this transformation is valid OpenMP code. Default \
                 is True
 
-        :returns: two-tuple of None and None.
-        :rtype: (None, None)
         '''
         self.validate(node, options=options)
 
@@ -488,6 +487,3 @@ class OMPTaskwaitTrans(Transformation):
                             for node1 in node_list):
                         task_region.dir_body.addchild(OMPTaskwaitDirective())
                         break
-
-        # TODO 595 return a tuple for now
-        return (None, None)
